@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2008 IBM Corporation and others.
+ * Copyright (c) 2000, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -31,7 +31,7 @@ import java.util.*;
  * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
 public final class Program {
-	String name;
+	String name = ""; //$NON-NLS-1$
 	String command;
 	String iconPath;
 	Display display;
@@ -74,7 +74,7 @@ static int getDesktop(final Display display) {
 	int desktop = DESKTOP_UNKNOWN;
 
 	/* Get the list of properties on the root window. */
-	long /*int*/ xDisplay = OS.GDK_DISPLAY();
+	long /*int*/ xDisplay = OS.gdk_x11_display_get_xdisplay(OS.gdk_display_get_default());
 	long /*int*/ rootWindow = OS.XDefaultRootWindow(xDisplay);
 	int[] numProp = new int[1];
 	long /*int*/ propList = OS.XListProperties(xDisplay, rootWindow, numProp);
@@ -94,29 +94,25 @@ static int getDesktop(final Display display) {
 	 * The workaround is to simply check that the window manager is a 
 	 * compliant one (property _NET_SUPPORTING_WM_CHECK) and to attempt to load 
 	 * our native library that depends on gnome-vfs.
+	 * 
+	 * Note: GIO is used when available instead of gnome-vfs.
 	 */
 	if (desktop == DESKTOP_UNKNOWN) {
 		byte[] gnomeName = Converter.wcsToMbcs(null, "_NET_SUPPORTING_WM_CHECK", true);
 		long /*int*/ gnome = OS.XInternAtom(xDisplay, gnomeName, true);
-		if (gnome != OS.None && (OS.GTK_VERSION >= OS.VERSION (2, 2, 0)) && gnome_init()) {
-			desktop = DESKTOP_GNOME;
-			long /*int*/ icon_theme = GNOME.gnome_icon_theme_new();
-			display.setData(ICON_THEME_DATA, new LONG(icon_theme));
-			display.addListener(SWT.Dispose, new Listener() {
-				public void handleEvent(Event event) {
-					LONG gnomeIconTheme = (LONG)display.getData(ICON_THEME_DATA);
-					if (gnomeIconTheme == null) return;
-					display.setData(ICON_THEME_DATA, null);
-					/* 
-					 * Note.  gnome_icon_theme_new uses g_object_new to allocate the
-					 * data it returns. Use g_object_unref to free the pointer it returns.
-					 */
-					if (gnomeIconTheme.value != 0) OS.g_object_unref(gnomeIconTheme.value);
-				}
-			});
-			/* Check for the existence of libgio libraries */
-			byte[] buffer = Converter.wcsToMbcs(null, "libgio-2.0.so.0", true);
-			long /*int*/ libgio = OS.dlopen(buffer, OS.RTLD_LAZY);
+		if (gnome != OS.None) {
+			/* Check for the existence of libgio libraries first */
+			byte[] buffer;
+			int flags = OS.RTLD_LAZY;
+			if (OS.IsAIX) {
+				buffer = Converter.wcsToMbcs(null, "libgio-2.0.a(libgio-2.0.so.0)", true);
+				flags |= OS.RTLD_MEMBER;
+			} else  if (OS.IsHPUX) {
+				buffer = Converter.wcsToMbcs(null, "libgio-2.0.so", true);
+			} else {
+				buffer =  Converter.wcsToMbcs(null, "libgio-2.0.so.0", true);
+			}
+			long /*int*/ libgio = OS.dlopen(buffer, flags);
 			if (libgio != 0) {
 				buffer = Converter.wcsToMbcs(null, "g_app_info_launch_default_for_uri", true);
 				long /*int*/ g_app_info_launch_default_for_uri = OS.dlsym(libgio, buffer);
@@ -124,7 +120,24 @@ static int getDesktop(final Display display) {
 					desktop = DESKTOP_GIO;
 				}
 				OS.dlclose(libgio);
-			} else {
+			}
+			
+			if (desktop == DESKTOP_UNKNOWN && (OS.GTK_VERSION >= OS.VERSION (2, 2, 0)) && gnome_init()) {
+				desktop = DESKTOP_GNOME;
+				long /*int*/ icon_theme = GNOME.gnome_icon_theme_new();
+				display.setData(ICON_THEME_DATA, new LONG(icon_theme));
+				display.addListener(SWT.Dispose, new Listener() {
+					public void handleEvent(Event event) {
+						LONG gnomeIconTheme = (LONG)display.getData(ICON_THEME_DATA);
+						if (gnomeIconTheme == null) return;
+						display.setData(ICON_THEME_DATA, null);
+						/* 
+						 * Note.  gnome_icon_theme_new uses g_object_new to allocate the
+						 * data it returns. Use g_object_unref to free the pointer it returns.
+						 */
+						if (gnomeIconTheme.value != 0) OS.g_object_unref(gnomeIconTheme.value);
+					}
+				});
 				/* Check for libgnomevfs-2 version 2.4 */
 				buffer = Converter.wcsToMbcs(null, "libgnomevfs-2.so.0", true);
 				long /*int*/ libgnomevfs = OS.dlopen(buffer, OS.RTLD_LAZY);
@@ -276,10 +289,13 @@ static String cde_getMimeType(String extension) {
 }
 
 static Program cde_getProgram(Display display, String mimeType) {
+	String command = cde_getAction(mimeType);
+	if (command == null) return null;
+
 	Program program = new Program();
 	program.display = display;
 	program.name = mimeType;
-	program.command = cde_getAction(mimeType);
+	program.command = command;
 	program.iconPath = cde_getAttribute(program.name, CDE.DtDTS_DA_ICON);
 	return program;
 }
@@ -294,7 +310,7 @@ static boolean cde_init(Display display) {
 	/* Use the character encoding for the default locale */
 	CDE.XtToolkitInitialize();
 	long /*int*/ xtContext = CDE.XtCreateApplicationContext ();
-	long /*int*/ xDisplay = OS.GDK_DISPLAY();
+	long /*int*/ xDisplay = OS.gdk_x11_display_get_xdisplay(OS.gdk_display_get_default());
 	byte[] appName = Converter.wcsToMbcs(null, "CDE", true);
 	byte[] appClass = Converter.wcsToMbcs(null, "CDE", true);
 	long /*int*/ [] argc = new long /*int*/ [] {0};
@@ -519,14 +535,18 @@ static Program gnome_getProgram(Display display, String mimeType) {
 		program.name = mimeType;
 		GnomeVFSMimeApplication application = new GnomeVFSMimeApplication();
 		GNOME.memmove(application, ptr, GnomeVFSMimeApplication.sizeof);
-		int length = OS.strlen(application.command);
-		byte[] buffer = new byte[length];
-		OS.memmove(buffer, application.command, length);		
-		program.command = new String(Converter.mbcsToWcs(null, buffer));
+		if (application.command != 0) {
+			int length = OS.strlen(application.command);
+			if (length > 0) {
+				byte[] buffer = new byte[length];
+				OS.memmove(buffer, application.command, length);		
+				program.command = new String(Converter.mbcsToWcs(null, buffer));
+			}
+		}
 		program.gnomeExpectUri = application.expects_uris == GNOME.GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS;
 		
-		length = OS.strlen(application.id);
-		buffer = new byte[length + 1];
+		int length = OS.strlen(application.id);
+		byte[] buffer = new byte[length + 1];
 		OS.memmove(buffer, application.id, length);
 		LONG gnomeIconTheme = (LONG)display.getData(ICON_THEME_DATA);
 		long /*int*/ icon_name = GNOME.gnome_icon_lookup(gnomeIconTheme.value, 0, null, buffer, 0, mimeTypeBuffer, 
@@ -545,7 +565,8 @@ static Program gnome_getProgram(Display display, String mimeType) {
 		if (icon_name != 0) GNOME.g_free(icon_name);
 		GNOME.gnome_vfs_mime_application_free(ptr);
 	}
-	return program;
+	
+	return program != null && program.command != null ? program : null;
 }
 
 static boolean gnome_init() {
@@ -882,7 +903,7 @@ static Program gio_getProgram (Display display, long /*int*/ application) {
 		}
 		OS.g_object_unref(icon);
 	}
-	return program;
+	return program.command != null ? program : null;
 }
 
 static Program[] gio_getPrograms(Display display) {
@@ -941,7 +962,7 @@ static boolean gio_isExecutable(String fileName) {
 static boolean gio_launch(String fileName) {
 	boolean result = false;
 	byte[] fileNameBuffer = Converter.wcsToMbcs (null, fileName, true);
-	long /*int*/ file = OS.g_file_new_for_path (fileNameBuffer);
+	long /*int*/ file = OS.g_file_new_for_commandline_arg (fileNameBuffer);
 	if (file != 0) {
 		long /*int*/ uri = OS.g_file_get_uri (file);
 		if (uri != 0) {
@@ -965,14 +986,17 @@ boolean gio_execute(String fileName) {
 	if (application != 0) {
 		byte[] fileNameBuffer = Converter.wcsToMbcs (null, fileName, true);
 		long /*int*/ file = 0;
-		if (OS.g_app_info_supports_uris (application)) {
-			file = OS.g_file_new_for_uri (fileNameBuffer);
-		} else {
-			file = OS.g_file_new_for_path (fileNameBuffer);
+		if (fileName.length() > 0) {
+			if (OS.g_app_info_supports_uris (application)) {
+				file = OS.g_file_new_for_uri (fileNameBuffer);
+			} else {
+				file = OS.g_file_new_for_path (fileNameBuffer);
+			}
 		}
-		if (file != 0) {
-			long /*int*/ list = OS.g_list_append (0, file);
-			result = OS.g_app_info_launch (application, list, 0, 0);
+		long /*int*/ list = 0;
+		if (file != 0) list = OS.g_list_append (0, file);
+		result = OS.g_app_info_launch (application, list, 0, 0);
+		if (list != 0) {
 			OS.g_list_free (list);
 			OS.g_object_unref (file);
 		}
