@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 IBM Corporation and others.
+ * Copyright (c) 2010, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,19 +11,26 @@
 
 package org.eclipse.ui.internal.handlers;
 
+import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
+import javax.inject.Named;
 import org.eclipse.core.commands.Command;
 import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
+import org.eclipse.core.commands.HandlerEvent;
 import org.eclipse.core.commands.IHandler;
 import org.eclipse.core.commands.NotEnabledException;
 import org.eclipse.core.commands.NotHandledException;
 import org.eclipse.core.commands.ParameterizedCommand;
 import org.eclipse.core.commands.common.NotDefinedException;
 import org.eclipse.core.expressions.ElementHandler;
+import org.eclipse.core.expressions.EvaluationContext;
 import org.eclipse.core.expressions.Expression;
 import org.eclipse.core.expressions.ExpressionConverter;
 import org.eclipse.core.expressions.IEvaluationContext;
@@ -35,32 +42,43 @@ import org.eclipse.e4.core.commands.ECommandService;
 import org.eclipse.e4.core.commands.EHandlerService;
 import org.eclipse.e4.core.commands.internal.HandlerServiceImpl;
 import org.eclipse.e4.core.contexts.ContextFunction;
-import org.eclipse.e4.core.contexts.IContextConstants;
+import org.eclipse.e4.core.contexts.EclipseContextFactory;
 import org.eclipse.e4.core.contexts.IEclipseContext;
+import org.eclipse.e4.core.di.InjectionException;
+import org.eclipse.e4.core.di.annotations.Execute;
+import org.eclipse.e4.core.di.annotations.Optional;
 import org.eclipse.e4.ui.internal.workbench.Activator;
 import org.eclipse.e4.ui.internal.workbench.Policy;
-import org.eclipse.e4.ui.internal.workbench.swt.AbstractPartRenderer;
-import org.eclipse.e4.ui.model.application.ui.MContext;
-import org.eclipse.swt.widgets.Control;
-import org.eclipse.swt.widgets.Display;
+import org.eclipse.e4.ui.workbench.modeling.ExpressionContext;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.ui.ISourceProvider;
+import org.eclipse.ui.ISources;
+import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchWindow;
-import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.handlers.IHandlerActivation;
 import org.eclipse.ui.handlers.IHandlerService;
+import org.eclipse.ui.internal.MakeHandlersGo;
+import org.eclipse.ui.internal.WorkbenchPlugin;
 import org.eclipse.ui.internal.e4.compatibility.E4Util;
 import org.eclipse.ui.internal.expressions.AndExpression;
 import org.eclipse.ui.internal.expressions.WorkbenchWindowExpression;
 import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;
-import org.eclipse.ui.internal.services.LegacyEvalContext;
+import org.eclipse.ui.internal.services.EvaluationService;
+import org.eclipse.ui.services.IEvaluationService;
+import org.eclipse.ui.services.ISourceProviderService;
 
 /**
  * @since 3.5
  * 
  */
 public class LegacyHandlerService implements IHandlerService {
-	final static String LEGACY_H_ID = "legacy::handler::"; //$NON-NLS-1$
+
+	private static final String[] SELECTION_VARIABLES = { ISources.ACTIVE_CURRENT_SELECTION_NAME,
+			ISources.ACTIVE_FOCUS_CONTROL_ID_NAME, ISources.ACTIVE_FOCUS_CONTROL_NAME,
+			ISources.ACTIVE_MENU_EDITOR_INPUT_NAME, ISources.ACTIVE_MENU_NAME,
+			ISources.ACTIVE_MENU_SELECTION_NAME };
+
+	public final static String LEGACY_H_ID = "legacy::handler::"; //$NON-NLS-1$
 
 	static class HandlerSelectionFunction extends ContextFunction {
 
@@ -76,18 +94,28 @@ public class LegacyHandlerService implements IHandlerService {
 		@Override
 		public Object compute(IEclipseContext context) {
 
-			List<HandlerActivation> handlerActivations = (List<HandlerActivation>) context
-					.get(LEGACY_H_ID + commandId);
+			HashSet<HandlerActivation> activationSet = new HashSet<HandlerActivation>();
+			IEclipseContext current = context;
+			while (current != null) {
+				List<HandlerActivation> handlerActivations = (List<HandlerActivation>) current
+						.getLocal(LEGACY_H_ID + commandId);
+				if (handlerActivations != null) {
+					activationSet.addAll(handlerActivations);
+				}
+				current = current.getParent();
+			}
 
-			if (handlerActivations == null) {
+			if (activationSet.isEmpty()) {
 				return null;
 			}
 
 			HandlerActivation bestActivation = null;
 
-			LegacyEvalContext legacyEvalContext = new LegacyEvalContext(context);
+			ExpressionContext legacyEvalContext = new ExpressionContext(context);
 
-			for (HandlerActivation handlerActivation : handlerActivations) {
+			HandlerActivation conflictBest = null;
+			HandlerActivation conflictOther = null;
+			for (HandlerActivation handlerActivation : activationSet) {
 				if (!handlerActivation.participating)
 					continue;
 				if (handlerActivation.evaluate(legacyEvalContext)) {
@@ -97,34 +125,116 @@ public class LegacyHandlerService implements IHandlerService {
 						int comparison = bestActivation.compareTo(handlerActivation);
 						if (comparison < 0) {
 							bestActivation = handlerActivation;
+						} else if (comparison == 0) {
+							conflictBest = bestActivation;
+							conflictOther = handlerActivation;
 						}
 					}
 				}
 			}
 
 			if (bestActivation != null) {
+				if (bestActivation == conflictBest) {
+					WorkbenchPlugin.log("Conflicting handlers for " + commandId + ": {" //$NON-NLS-1$ //$NON-NLS-2$
+							+ conflictBest.getHandler() + "} vs {" //$NON-NLS-1$
+							+ conflictOther.getHandler() + "}"); //$NON-NLS-1$
+				}
 				return bestActivation.proxy;
 			}
 
-			// "super call"
-			IEclipseContext parent = context.getParent();
-			if (parent == null) {
-				return null;
-			}
-			return parent.get(HandlerServiceImpl.H_ID + commandId);
+			return null;
 		}
 	}
 
+	private static IHandlerActivation systemHandlerActivation;
+
+	/*
+	 * We are obligated to return a non-null IHandlerActivation from our
+	 * activate calls. It is only used as a token, but must not return null from
+	 * certain methods. This token represents passing the MakeHandlerGo handler
+	 * back into the system.
+	 */
+	private static IHandlerActivation getSystemHandlerActivation(IEclipseContext context, final String cmdId) {
+		if (systemHandlerActivation == null) {
+			final IWorkbench wb = context.get(IWorkbench.class);
+
+			systemHandlerActivation = new IHandlerActivation() {
+
+				public int compareTo(Object o) {
+					return -1;
+				}
+
+				public void setResult(boolean result) {
+				}
+
+				public int getSourcePriority() {
+					return 0;
+				}
+
+				public Expression getExpression() {
+					return null;
+				}
+
+				public boolean evaluate(IEvaluationContext context) {
+					return false;
+				}
+
+				public void clearResult() {
+				}
+
+				public boolean isActive(IEvaluationContext context) {
+					return false;
+				}
+
+				public IHandlerService getHandlerService() {
+					return (IHandlerService) wb.getService(IHandlerService.class);
+				}
+
+				public IHandler getHandler() {
+					return null;
+				}
+
+				public int getDepth() {
+					return 0;
+				}
+
+				public String getCommandId() {
+					return cmdId;
+				}
+
+				public void clearActive() {
+				}
+			};
+		}
+		return systemHandlerActivation;
+	}
+
 	public static IHandlerActivation registerLegacyHandler(final IEclipseContext context,
 			String id, final String cmdId, IHandler handler, Expression activeWhen) {
+		if (handler instanceof MakeHandlersGo) {
+			final String msg = "Invalid Handler MakeHandlerGo"; //$NON-NLS-1$
+			WorkbenchPlugin.log(msg, new Exception(msg));
+			return getSystemHandlerActivation(context, cmdId);
+		}
 		ECommandService cs = (ECommandService) context.get(ECommandService.class.getName());
 		Command command = cs.getCommand(cmdId);
+		boolean handled = command.isHandled();
+		boolean enabled = command.isEnabled();
 		E4HandlerProxy handlerProxy = new E4HandlerProxy(command, handler);
 		HandlerActivation activation = new HandlerActivation(context, cmdId, handler, handlerProxy,
 				activeWhen);
 		addHandlerActivation(activation);
 		EHandlerService hs = context.get(EHandlerService.class);
 		hs.activateHandler(cmdId, new HandlerSelectionFunction(cmdId));
+		boolean handledChanged = handled != command.isHandled();
+		boolean enabledChanged = enabled != command.isEnabled();
+		if (handledChanged || enabledChanged) {
+			IHandler proxy = command.getHandler();
+			if (proxy instanceof MakeHandlersGo) {
+				((MakeHandlersGo) proxy).fireHandlerChanged(new HandlerEvent(proxy, enabledChanged,
+						handledChanged));
+			}
+		}
 		return activation;
 	}
 
@@ -134,6 +244,9 @@ public class LegacyHandlerService implements IHandlerService {
 		if (handlerActivations == null) {
 			handlerActivations = new ArrayList();
 		} else {
+			if (handlerActivations.contains(eActivation)) {
+				return;
+			}
 			handlerActivations = new ArrayList(handlerActivations);
 		}
 		handlerActivations.add(eActivation);
@@ -160,7 +273,7 @@ public class LegacyHandlerService implements IHandlerService {
 
 	public LegacyHandlerService(IEclipseContext context) {
 		eclipseContext = context;
-		evalContext = new LegacyEvalContext(eclipseContext);
+		evalContext = new ExpressionContext(eclipseContext);
 		IWorkbenchWindow window = (IWorkbenchWindow) eclipseContext.get(IWorkbenchWindow.class
 				.getName());
 		if (window != null) {
@@ -168,6 +281,36 @@ public class LegacyHandlerService implements IHandlerService {
 		}
 	}
 
+	public LegacyHandlerService(IEclipseContext context, Expression defaultExpression) {
+		eclipseContext = context;
+		evalContext = new ExpressionContext(eclipseContext);
+		this.defaultExpression = defaultExpression;
+	}
+
+	public void initPreExecuteHook() {
+		EHandlerService hs = eclipseContext.get(EHandlerService.class);
+		if (hs instanceof HandlerServiceImpl) {
+			HandlerServiceImpl.preExecute = new Object() {
+				@Execute
+				public void execute(IEclipseContext context, ParameterizedCommand command,
+						@Optional @Named(HandlerServiceImpl.PARM_MAP) Map parms,
+						@Optional Event trigger, @Optional IEvaluationContext staticContext) {
+					if (command == null) {
+						return;
+					}
+					IEvaluationContext appContext = staticContext;
+					if (appContext == null) {
+						appContext = new ExpressionContext(context);
+					}
+					ExecutionEvent event = new ExecutionEvent(command.getCommand(), parms, trigger,
+							appContext);
+					CommandProxy.firePreExecute(command.getCommand(), event);
+				}
+			};
+		}
+
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -209,6 +352,9 @@ public class LegacyHandlerService implements IHandlerService {
 	 * .handlers.IHandlerActivation)
 	 */
 	public IHandlerActivation activateHandler(IHandlerActivation activation) {
+		if (activation == systemHandlerActivation) {
+			return activation;
+		}
 		HandlerActivation handlerActivation = (HandlerActivation) activation;
 		handlerActivation.participating = true;
 		addHandlerActivation(handlerActivation);
@@ -279,8 +425,8 @@ public class LegacyHandlerService implements IHandlerService {
 	 * .core.commands.Command, org.eclipse.swt.widgets.Event)
 	 */
 	public ExecutionEvent createExecutionEvent(Command command, Event event) {
-		LegacyEvalContext legacy = new LegacyEvalContext(getFocusContext(PlatformUI.getWorkbench()
-				.getDisplay()));
+		EvaluationContext legacy = new EvaluationContext(evalContext,
+				evalContext.getDefaultVariable());
 		ExecutionEvent e = new ExecutionEvent(command, Collections.EMPTY_MAP, event, legacy);
 		return e;
 	}
@@ -293,8 +439,8 @@ public class LegacyHandlerService implements IHandlerService {
 	 * .core.commands.ParameterizedCommand, org.eclipse.swt.widgets.Event)
 	 */
 	public ExecutionEvent createExecutionEvent(ParameterizedCommand command, Event event) {
-		LegacyEvalContext legacy = new LegacyEvalContext(getFocusContext(PlatformUI.getWorkbench()
-				.getDisplay()));
+		EvaluationContext legacy = new EvaluationContext(evalContext,
+				evalContext.getDefaultVariable());
 		ExecutionEvent e = new ExecutionEvent(command.getCommand(), command.getParameterMap(),
 				event, legacy);
 		return e;
@@ -308,9 +454,12 @@ public class LegacyHandlerService implements IHandlerService {
 	 * .ui.handlers.IHandlerActivation)
 	 */
 	public void deactivateHandler(IHandlerActivation activation) {
-		HandlerActivation eActivation = (HandlerActivation) activation;
-		eActivation.participating = false;
-		removeHandlerActivation(eActivation);
+		// null is not allowed, but some people put it anyway :( see bug 326406
+		if (activation != null && activation != systemHandlerActivation) {
+			HandlerActivation eActivation = (HandlerActivation) activation;
+			eActivation.participating = false;
+			removeHandlerActivation(eActivation);
+		}
 	}
 
 	/*
@@ -342,8 +491,7 @@ public class LegacyHandlerService implements IHandlerService {
 	 */
 	public Object executeCommand(String commandId, Event event) throws ExecutionException,
 			NotDefinedException, NotEnabledException, NotHandledException {
-		ECommandService cs = (ECommandService) getFocusContext(
-				PlatformUI.getWorkbench().getDisplay()).get(ECommandService.class.getName());
+		ECommandService cs = eclipseContext.get(ECommandService.class);
 		final Command command = cs.getCommand(commandId);
 		return executeCommand(ParameterizedCommand.generateCommand(command, null), event);
 	}
@@ -358,9 +506,35 @@ public class LegacyHandlerService implements IHandlerService {
 	public Object executeCommand(ParameterizedCommand command, Event event)
 			throws ExecutionException, NotDefinedException, NotEnabledException,
 			NotHandledException {
-		EHandlerService hs = (EHandlerService) getFocusContext(
-				PlatformUI.getWorkbench().getDisplay()).get(EHandlerService.class.getName());
-		return hs.executeHandler(command);
+		EHandlerService hs = eclipseContext.get(EHandlerService.class);
+		IEclipseContext staticContext = EclipseContextFactory.create();
+		if (event != null) {
+			staticContext.set(Event.class, event);
+		}
+		try {
+			final Object rc = hs.executeHandler(command, staticContext);
+			if (staticContext.get(HandlerServiceImpl.NOT_HANDLED) == Boolean.TRUE) {
+				final NotHandledException e = new NotHandledException(
+						"There is no handler to execute for command " + command.getId()); //$NON-NLS-1$
+				CommandProxy.fireNotHandled(command.getCommand(), e);
+				throw e;
+			}
+			final Object obj = staticContext.get(HandlerServiceImpl.CAN_EXECUTE);
+			if (obj instanceof Boolean) {
+				if (!((Boolean) obj).booleanValue()) {
+					final NotEnabledException exception = new NotEnabledException(
+							"Trying to execute the disabled command " + command.getId()); //$NON-NLS-1$
+					CommandProxy.fireNotEnabled(command.getCommand(), exception);
+					throw exception;
+				}
+			}
+			return rc;
+		} catch (InjectionException e) {
+			rethrow(e);
+			throw e;
+		} finally {
+			staticContext.dispose();
+		}
 	}
 
 	/*
@@ -374,12 +548,123 @@ public class LegacyHandlerService implements IHandlerService {
 	public Object executeCommandInContext(ParameterizedCommand command, Event event,
 			IEvaluationContext context) throws ExecutionException, NotDefinedException,
 			NotEnabledException, NotHandledException {
-		if (context instanceof LegacyEvalContext) {
-			EHandlerService hs = (EHandlerService) ((LegacyEvalContext) context).eclipseContext
-					.get(EHandlerService.class.getName());
-			return hs.executeHandler(command);
+
+		IEclipseContext staticContext = null;
+		Object defaultVar = null;
+		if (context instanceof ExpressionContext) {
+			// create a child context so that the primary context doesn't get
+			// populated by parameters by the EHS
+			staticContext = ((ExpressionContext) context).eclipseContext.createChild();
+		} else {
+			staticContext = eclipseContext.getActiveLeaf().createChild("snapshotContext"); //$NON-NLS-1$
+			if (event != null) {
+				staticContext.set(Event.class, event);
+			}
+			staticContext.set(IEvaluationContext.class, context);
+			defaultVar = context.getDefaultVariable();
+			if (defaultVar != null && defaultVar != IEvaluationContext.UNDEFINED_VARIABLE) {
+				staticContext.set(EvaluationService.DEFAULT_VAR, defaultVar);
+			}
+		}
+		IEclipseContext lookupContext = staticContext;
+		// the IEvaluationContext snapshot is not part of our runtime
+		// IEclipseContext hierarchy. In order to work. we need the variables
+		// from the snapshot available in the static context.
+		populateSnapshot(context, staticContext);
+		EHandlerService hs = lookupContext.get(EHandlerService.class);
+		try {
+			final Object rc = hs.executeHandler(command, staticContext);
+			if (staticContext.get(HandlerServiceImpl.NOT_HANDLED) == Boolean.TRUE) {
+				final NotHandledException e = new NotHandledException(
+						"There is no handler to execute for command " + command.getId()); //$NON-NLS-1$
+				CommandProxy.fireNotHandled(command.getCommand(), e);
+				throw e;
+			}
+			final Object obj = staticContext.get(HandlerServiceImpl.CAN_EXECUTE);
+			if (obj instanceof Boolean) {
+				if (!((Boolean) obj).booleanValue()) {
+					final NotEnabledException exception = new NotEnabledException(
+							"Trying to execute the disabled command " + command.getId()); //$NON-NLS-1$
+					CommandProxy.fireNotEnabled(command.getCommand(), exception);
+					throw exception;
+				}
+			}
+			return rc;
+		} catch (InjectionException e) {
+			rethrow(e);
+			throw e;
+		} finally {
+			staticContext.dispose();
+		}
+	}
+
+	/**
+	 * @param context
+	 * @param staticContext
+	 */
+	private void populateSnapshot(IEvaluationContext context, IEclipseContext staticContext) {
+		IEvaluationContext ctxPtr = context;
+		while (ctxPtr != null && !(ctxPtr instanceof ExpressionContext)) {
+			Map vars = getVariables(ctxPtr);
+			if (vars != null) {
+				Iterator i = vars.entrySet().iterator();
+				while (i.hasNext()) {
+					Map.Entry entry = (Map.Entry) i.next();
+					if (staticContext.getLocal(entry.getKey().toString()) == null) {
+						staticContext.set(entry.getKey().toString(), entry.getValue());
+					}
+				}
+			}
+			ctxPtr = ctxPtr.getParent();
+		}
+	}
+
+	private Map getVariables(IEvaluationContext ctx) {
+		Field vars = getContextVariablesField();
+		if (vars != null) {
+			try {
+				return (Map) vars.get(ctx);
+			} catch (IllegalArgumentException e) {
+
+			} catch (IllegalAccessException e) {
+
+			}
+		}
+		return null;
+	}
+
+	private static Field contextFVariables = null;
+
+	private static Field getContextVariablesField() {
+		if (contextFVariables == null) {
+			try {
+				contextFVariables = EvaluationContext.class.getField("fVariables"); //$NON-NLS-1$
+				contextFVariables.setAccessible(true);
+			} catch (SecurityException e) {
+
+			} catch (NoSuchFieldException e) {
+
+			}
+		}
+		return contextFVariables;
+	}
+
+	/**
+	 * Checks the cause of the provided exception and rethrows the cause instead
+	 * if it was one of the expected exception types.
+	 */
+	private void rethrow(InjectionException e) throws ExecutionException, NotDefinedException,
+			NotEnabledException, NotHandledException {
+		Throwable cause = e.getCause();
+		if (cause instanceof ExecutionException) {
+			throw (ExecutionException) cause;
+		} else if (cause instanceof NotDefinedException) {
+			throw (NotDefinedException) cause;
+		} else if (cause instanceof NotEnabledException) {
+			throw (NotEnabledException) cause;
+		} else if (cause instanceof NotHandledException) {
+			throw (NotHandledException) cause;
 		}
-		return executeCommand(command, event);
 	}
 
 	/*
@@ -389,12 +674,43 @@ public class LegacyHandlerService implements IHandlerService {
 	 * org.eclipse.ui.handlers.IHandlerService#createContextSnapshot(boolean)
 	 */
 	public IEvaluationContext createContextSnapshot(boolean includeSelection) {
-		IEclipseContext targetContext = eclipseContext;
-		while (targetContext.getLocal(IContextConstants.ACTIVE_CHILD) != null) {
-			targetContext = (IEclipseContext) targetContext
-					.getLocal(IContextConstants.ACTIVE_CHILD);
+		IEvaluationContext tmpContext = getCurrentState();
+		IEvaluationContext context = new EvaluationContext(null,
+				IEvaluationContext.UNDEFINED_VARIABLE);
+
+		if (includeSelection) {
+			for (String variable : SELECTION_VARIABLES) {
+				copyVariable(context, tmpContext, variable);
+			}
+		}
+
+		ISourceProviderService sp = eclipseContext.get(ISourceProviderService.class);
+		for (ISourceProvider provider : sp.getSourceProviders()) {
+			String[] names = provider.getProvidedSourceNames();
+			for (String name : names) {
+				if (!isSelectionVariable(name)) {
+					copyVariable(context, tmpContext, name);
+				}
+			}
+		}
+
+		return context;
+	}
+
+	private boolean isSelectionVariable(String name) {
+		for (String variable : SELECTION_VARIABLES) {
+			if (variable.equals(name)) {
+				return true;
+			}
+		}
+		return false;
+	}
+
+	private void copyVariable(IEvaluationContext context, IEvaluationContext tmpContext, String var) {
+		Object o = tmpContext.getVariable(var);
+		if (o != null) {
+			context.addVariable(var, o);
 		}
-		return new LegacyEvalContext(targetContext.createChild());
 	}
 
 	/*
@@ -403,7 +719,7 @@ public class LegacyHandlerService implements IHandlerService {
 	 * @see org.eclipse.ui.handlers.IHandlerService#getCurrentState()
 	 */
 	public IEvaluationContext getCurrentState() {
-		return evalContext;
+		return new EvaluationContext(evalContext, evalContext.getDefaultVariable());
 	}
 
 	/*
@@ -453,9 +769,32 @@ public class LegacyHandlerService implements IHandlerService {
 							"Incorrect activeWhen element " + commandId, e); //$NON-NLS-1$
 				}
 			}
-			registerLegacyHandler(eclipseContext, commandId, commandId,
+			Expression enabledWhen = null;
+			final IConfigurationElement[] ewChildren = configElement
+					.getChildren(IWorkbenchRegistryConstants.TAG_ENABLED_WHEN);
+			if (ewChildren.length > 0) {
+				final IConfigurationElement[] subChildren = ewChildren[0].getChildren();
+				if (subChildren.length != 1) {
+					Activator.trace(Policy.DEBUG_CMDS,
+							"Incorrect enableWhen element " + commandId, null); //$NON-NLS-1$
+					continue;
+				}
+				final ElementHandler elementHandler = ElementHandler.getDefault();
+				final ExpressionConverter converter = ExpressionConverter.getDefault();
+				try {
+					enabledWhen = elementHandler.create(converter, subChildren[0]);
+				} catch (CoreException e) {
+					Activator.trace(Policy.DEBUG_CMDS,
+							"Incorrect enableWhen element " + commandId, e); //$NON-NLS-1$
+				}
+			}
+			registerLegacyHandler(
+					eclipseContext,
+					commandId,
+					commandId,
 					new org.eclipse.ui.internal.handlers.HandlerProxy(commandId, configElement,
-							IWorkbenchRegistryConstants.ATT_CLASS), activeWhen);
+							IWorkbenchRegistryConstants.ATT_CLASS, enabledWhen, eclipseContext
+									.get(IEvaluationService.class)), activeWhen);
 		}
 	}
 
@@ -494,18 +833,4 @@ public class LegacyHandlerService implements IHandlerService {
 		// TODO Auto-generated method stub
 
 	}
-
-	private IEclipseContext getFocusContext(Display display) {
-		// find the first useful part in the model
-		Control control = display.getFocusControl();
-		Object modelObj = null;
-		while (control != null) {
-			modelObj = control.getData(AbstractPartRenderer.OWNING_ME);
-			if (modelObj instanceof MContext)
-				return ((MContext) modelObj).getContext();
-			control = control.getParent();
-		}
-		return eclipseContext;
-	}
-
 }
